from django.contrib.postgres.fields import JSONField

from django_jsonschema.validators import JSONSchemaValidator


class JSONSchemaField(JSONField):
    def __init__(self, schema='', **kwargs):
        self.schema = schema
        super(JSONSchemaField, self).__init__(**kwargs)
        self.validators.append(JSONSchemaValidator(self.schema))

    def from_db_value(self, value, expression, connection, context):
        if value is not None:
            self.run_validators(value)
        return value

    def get_prep_value(self, value):
        if value is not None:
            self.run_validators(value)
        return super(JSONSchemaField, self).get_prep_value(value)

    def get_prep_lookup(self, lookup_type, value):
        if lookup_type in {
            'iexact', 'contains', 'icontains',
            'startswith', 'istartswith', 'endswith', 'iendswith',
            'isnull', 'search', 'regex', 'iregex',
            'has_key', 'has_keys', 'has_any_keys'
        }:
            return value

        if isinstance(value, (dict, list)):
            return Json(value)

        if hasattr(value, '_prepare'):
            return value._prepare()

        get_prep_value = super(JSONSchemaField, self).get_prep_value
        if lookup_type in ('exact', 'gt', 'gte', 'lt', 'lte'):
            return get_prep_value(value)
        elif lookup_type in ('range', 'in'):
            return [get_prep_value(v) for v in value]
        return get_prep_value(value)



from django.core.exceptions import ValidationError
from jsonschema import validate, ValidationError as JSONSchemaValidationError


class JSONSchemaValidator(object):
    code = 'invalid'

    def __init__(self, schema):
        self.schema = schema

    def __call__(self, value):
        try:
            validate(value, self.schema)
        except JSONSchemaValidationError as e:
            raise ValidationError(
                '%s: %s' % ('.'.join(e.path), e.message), code=self.code,)
        else:
            return value

    def __eq__(self, other):
        return (
            isinstance(other, self.__class__) and self.schema == other.schema
        )



import json

from django import forms
from jsonschema import ValidationError

from .validators import JSONSchemaValidator


class JSONSchemaField(forms.CharField):
    def __init__(self, schema, **kwargs):
        self.schema = schema
        super(JSONSchemaField, self).__init__(**kwargs)
        self.validators.append(JSONSchemaValidator(self.schema))

    def to_python(self, value):
        try:
            value = json.loads(value)
        except ValueError as error:
            raise ValidationError(str(error))
        return value



import json

from django.db import models

from .validators import JSONSchemaValidator


class JSONSchemaField(models.TextField):
    def __init__(self, schema='', **kwargs):
        self.schema = schema
        super(JSONSchemaField, self).__init__(**kwargs)
        self.validators.append(JSONSchemaValidator(self.schema))

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return
        value = json.loads(value)
        self.run_validators(value)
        return value

    def get_prep_value(self, value):
        if value is None:
            return
        self.run_validators(value)
        return json.dumps(value)



import os
from setuptools import setup, find_packages

VERSION = '0.2.1'
PATH = os.path.dirname(os.path.abspath(__file__))


def read(fname):
    return open(os.path.join(PATH, fname)).read()


install_requires = [
    'jsonschema',
    'django',
]

setup(
    name='django-jsonschema2',
    version=VERSION,
    description='Django JSONSchema',
    long_description=read('README.md'),
    url='https://github.com/hypc/django-jsonschema',
    author='hypc',
    author_email='h_yp00@163.com',
    license='MIT',
    packages=find_packages(exclude=['docs', 'tests*', 'example']),
    install_requires=install_requires,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Intended Audience :: Developers',
        'Topic :: Internet',
    ]
)
